using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;
using submgr = Microsoft.Dss.Services.SubscriptionManager;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Text;

namespace Robotics.csprojUDP2
{
    [Contract(Contract.Identifier)]
    [DisplayName("csprojUDP2")]
    [Description("csprojUDP2 service (no description provided)")]
    class csprojUDP2Service : DsspServiceBase
    {
        private const int BUFSIZE = 32;
        private const String robotIP = "192.168.0.2";
        private const int servPort = 8000;
        byte[] rcvBuffer = new byte[BUFSIZE];
        UdpClient client = null;
        IPEndPoint remoteIPEndPoint = null;
        /// <summary>
        /// Service state
        /// </summary>
        [ServiceState]
        csprojUDP2State _state = new csprojUDP2State();

        /// <summary>
        /// Main service port
        /// </summary>
        [ServicePort("/csprojUDP2", AllowMultipleInstances = true)]
        csprojUDP2Operations _mainPort = new csprojUDP2Operations();

        [SubscriptionManagerPartner]
        submgr.SubscriptionManagerPort _submgrPort = new submgr.SubscriptionManagerPort();

        /// <summary>
        /// Service constructor
        /// </summary>
        public csprojUDP2Service(DsspServiceCreationPort creationPort)
            : base(creationPort)
        {
            _state.Name = "fineday";
            _state.Command = "null";
        }

        /// <summary>
        /// Service start
        /// </summary>
        protected override void Start()
        {

            // 
            // Add service specific initialization here
            // 

            base.Start();
            try
            {
                // NCAg̎t
                client = new UdpClient(servPort);
            }
            catch (SocketException se)
            {
                LogInfo(se.ErrorCode + ";" + se.Message);
            }
            // Gh|Cg̍쐬
            remoteIPEndPoint = new IPEndPoint(IPAddress.Any, servPort);

            // NCAg̃ANZX҂
            rcvBuffer = client.Receive(ref remoteIPEndPoint);
            // oCg當֕ϊ
            int bytesRcvd = rcvBuffer.GetLength(0);
            // string֕ϊ
            string str = Encoding.ASCII.GetString(rcvBuffer, 0, bytesRcvd - 1);
            LogInfo("Accessed from iPhone: " + str);

            // XbhX^[g
            receiveUDP receiveudp = new receiveUDP(remoteIPEndPoint, client, _mainPort);
            Thread thread = new Thread(new ThreadStart(receiveudp.receiveMsg));
            thread.Start();
        }

        /// <summary>
        /// Handles Subscribe messages
        /// </summary>
        /// <param name="subscribe">the subscribe request</param>
        [ServiceHandler]
        public void SubscribeHandler(Subscribe subscribe)
        {
            SubscribeHelper(_submgrPort, subscribe.Body, subscribe.ResponsePort);
        }
        /// <summary>
        /// Handles SetCommand messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> SetCommandHandler(SetCommand setCommand)
        {
            string str = setCommand.Body.Command;
            LogInfo("Received: " + str);
            _state.Command = str;
            if (str == "end" || str == "ERROR")
            {
                client.Close();
            }

            base.SendNotification(_submgrPort, new SetCommand(str));
//            base.SendNotification(_submgrPort, new SetCommand(_state.Command));

            setCommand.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            yield break;
        }
        /// <summary>
        /// Handles SendMsg messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> SendMsgHandler(SendMsg sendMsg)
        {
            byte[] rcvBuffer = new byte[BUFSIZE];
            rcvBuffer = Encoding.ASCII.GetBytes(sendMsg.Body.msgstr);
            client.Send(rcvBuffer, rcvBuffer.Length, remoteIPEndPoint);
            sendMsg.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            yield break;
        }
    }
}


