using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;
using submgr = Microsoft.Dss.Services.SubscriptionManager;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Text;

namespace Robotics.csprojTCP
{
    [Contract(Contract.Identifier)]
    [DisplayName("csprojTCP")]
    [Description("csprojTCP service (no description provided)")]
    class csprojTCPService : DsspServiceBase
    {
        private const int BUFSIZE = 32;
        private const String robotIP = "192.168.0.2";
        private const int servPort = 8000;
        private Socket clntSock;
        /// <summary>
        /// Service state
        /// </summary>
        [ServiceState]
        csprojTCPState _state = new csprojTCPState();

        /// <summary>
        /// Main service port
        /// </summary>
        [ServicePort("/csprojTCP", AllowMultipleInstances = true)]
        csprojTCPOperations _mainPort = new csprojTCPOperations();

        [SubscriptionManagerPartner]
        submgr.SubscriptionManagerPort _submgrPort = new submgr.SubscriptionManagerPort();

        /// <summary>
        /// Service constructor
        /// </summary>
        public csprojTCPService(DsspServiceCreationPort creationPort)
            : base(creationPort)
        {
            _state.Name = "fineday";
            _state.Command = "null";
        }

        /// <summary>
        /// Service start
        /// </summary>
        protected override void Start()
        {

            // 
            // Add service specific initialization here
            // 

            base.Start();

            // bXJn
            TcpListener listener = null;
            listener = new TcpListener(IPAddress.Any, servPort);
            listener.Start();

            // ANZvg
            clntSock = listener.AcceptSocket();
            LogInfo("iPhone accepted");

            // XbhX^[g
            receiveTCP receivetcp = new receiveTCP(clntSock, _mainPort);
            Thread thread = new Thread(new ThreadStart(receivetcp.receiveMsg));
            thread.Start();
        }

        /// <summary>
        /// Handles Subscribe messages
        /// </summary>
        /// <param name="subscribe">the subscribe request</param>
        [ServiceHandler]
        public void SubscribeHandler(Subscribe subscribe)
        {
            SubscribeHelper(_submgrPort, subscribe.Body, subscribe.ResponsePort);
        }
        /// <summary>
        /// Handles SetCommand messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> SetCommandHandler(SetCommand setCommand)
        {
            string str = setCommand.Body.Command;
            LogInfo("Received: " + str);
            _state.Command = str;
            if (str == "end" || str == "ERROR")
            {
                clntSock.Close();
            }
            setCommand.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            yield break;
        }
        /// <summary>
        /// Handles SendMsg messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> SendMsgHandler(SendMsg sendMsg)
        {
            byte[] rcvBuffer = new byte[BUFSIZE];
            rcvBuffer = Encoding.ASCII.GetBytes(sendMsg.Body.msgstr);
            clntSock.Send(rcvBuffer, 0, rcvBuffer.Length, SocketFlags.None);
            sendMsg.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            yield break;
        }
    }
}


