using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;

namespace Robotics.csprojTCP
{
    /// <summary>
    /// csprojTCP contract class
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// DSS contract identifer for csprojTCP
        /// </summary>
        [DataMember]
        public const string Identifier = "http://schemas.tempuri.org/2011/03/csprojtcp.html";
    }

    /// <summary>
    /// csprojTCP state
    /// </summary>
    [DataContract]
    public class csprojTCPState
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public string Command { get; set; }
    }

    /// <summary>
    /// csprojTCP main operations port
    /// </summary>
    [ServicePort]
    public class csprojTCPOperations : PortSet<DsspDefaultLookup, DsspDefaultDrop, Get, Subscribe, SetCommand, SendMsg>
    {
    }

    /// <summary>
    /// csprojTCP get operation
    /// </summary>
    public class Get : Get<GetRequestType, PortSet<csprojTCPState, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        public Get()
        {
        }

        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        /// <param name="body">the request message body</param>
        public Get(GetRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Get(GetRequestType body, PortSet<csprojTCPState, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }

    /// <summary>
    /// csprojTCP subscribe operation
    /// </summary>
    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        public Subscribe()
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        public Subscribe(SubscribeRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Subscribe(SubscribeRequestType body, PortSet<SubscribeResponseType, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }
    /// <summary>
    /// TCP SetCommand operation
    /// </summary>
    public class SetCommand : Update<SetCommandBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SetCommand() : base(new SetCommandBody()) { }
        public SetCommand(string command) : base(new SetCommandBody(command)) { }
    }
    // Increment Body
    [DataContract]
    public class SetCommandBody
    {
        // RXgN^
        public SetCommandBody()
        {
        }
        // RXgN^
        public SetCommandBody(string command)
        {
            Command = command;
        }
        [DataMember]
        public string Command { get; set; }
    }
    /// <summary>
    /// TCP SendMsg operation
    /// </summary>
    public class SendMsg : Update<SendMsgBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SendMsg() : base(new SendMsgBody()) { }
        public SendMsg(string msg) : base(new SendMsgBody(msg)) { }
    }
    // SendMsg Body
    [DataContract]
    public class SendMsgBody
    {
        // RXgN^
        public SendMsgBody() { }
        // RXgN^
        public SendMsgBody(string str)
        {
            msgstr = str;
        }
        [DataMember]
        public string msgstr { get; set; }
    }
}


