using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;
using submgr = Microsoft.Dss.Services.SubscriptionManager;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Text;

namespace Robotics.UDP701
{
    [Contract(Contract.Identifier)]
    [DisplayName("UDP701")]
    [Description("UDP701 service (no description provided)")]
    class UDP701Service : DsspServiceBase
    {
        private const int BUFSIZE = 32;
        private const String robotIP = "192.168.0.2";
        private const int servPort = 8000;
        private const char openCode = 'i';
        private const char packetCode = 'p';
        private const char shutdownCode = 'c';
        byte[] rcvBuffer = new byte[BUFSIZE];
        UdpClient client = null;
        IPEndPoint remoteIPEndPoint = null;
        Thread thread;
        /// <summary>
        /// Service state
        /// </summary>
        [ServiceState]
        UDP701State _state = new UDP701State();

        /// <summary>
        /// Main service port
        /// </summary>
        [ServicePort("/UDP701", AllowMultipleInstances = true)]
        UDP701Operations _mainPort = new UDP701Operations();

        [SubscriptionManagerPartner]
        submgr.SubscriptionManagerPort _submgrPort = new submgr.SubscriptionManagerPort();

        /// <summary>
        /// Service constructor
        /// </summary>
        public UDP701Service(DsspServiceCreationPort creationPort)
            : base(creationPort)
        {
            _state.Name = "fineday";
            _state.Command = '\n';
        }

        /// <summary>
        /// Service start
        /// </summary>
        protected override void Start()
        {

            // 
            // Add service specific initialization here
            // 

            base.Start();
            try
            {
                // NCAg̎t
                client = new UdpClient(servPort);
            }
            catch (SocketException se)
            {
                LogInfo(se.ErrorCode + ";" + se.Message);
            }
            // Gh|Cg̍쐬
            remoteIPEndPoint = new IPEndPoint(IPAddress.Any, servPort);

            // NCAg̃ANZX҂
            rcvBuffer = client.Receive(ref remoteIPEndPoint);
            // oCg當֕ϊ
            int bytesRcvd = rcvBuffer.GetLength(0);
            char cmd = (char)rcvBuffer[0];
            if (cmd == openCode)
            {
                LogInfo("iPhone accepted");
                // XbhX^[g
                receiveUDP receiveudp = new receiveUDP(remoteIPEndPoint, client, _mainPort);
                thread = new Thread(new ThreadStart(receiveudp.receiveMsg));
                thread.Start();
                LogInfo("thread started");
            }
            else
            {
                LogInfo("unknown code: System CLOSED");
            }
        }

        /// <summary>
        /// Handles Subscribe messages
        /// </summary>
        /// <param name="subscribe">the subscribe request</param>
        [ServiceHandler]
        public void SubscribeHandler(Subscribe subscribe)
        {
            SubscribeHelper(_submgrPort, subscribe.Body, subscribe.ResponsePort);
        }
        /// <summary>
        /// Handles SetCommand messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> SetCommandHandler(SetCommand setCommand)
        {
            rcvBuffer = setCommand.Body.Command;
            char cmd = (char)rcvBuffer[0];
            _state.Command = cmd;
            if (cmd == packetCode)
            {
                LogInfo("packet accepted");
                string s = Encoding.ASCII.GetString(rcvBuffer, 1, 10);
                float t = float.Parse(s);
                LogInfo("time = " + t);
                string s2 = Encoding.ASCII.GetString(rcvBuffer, 11, 10);
                //         int n = int.Parse(s2);
                float n = float.Parse(s2);
                LogInfo("number = " + n);
            }
            else if (cmd == shutdownCode)
            {
                LogInfo("ENDCODE accepted");
                thread.Abort();
                client.Close();
            }
            else
            {
                LogInfo("undefined code: Syetem CLOSED");
            }
            setCommand.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            yield break;
        }
        /// <summary>
        /// Handles SendMsg messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> SendMsgHandler(SendMsg sendMsg)
        {
            byte[] rcvBuffer = new byte[BUFSIZE];
            rcvBuffer = Encoding.ASCII.GetBytes(sendMsg.Body.msgstr);
            /*
            //            char c = 'd';
                        float x = 1234.89f;
            //            float y = 56.789f;
                        string s = System.String.Format("{0,8}",x);
            //            string s = System.String.Format("{0,1}{1,10}{2,10}", c, x, y);
                        rcvBuffer = Encoding.ASCII.GetBytes(s);
            */
            client.Send(rcvBuffer, rcvBuffer.Length, remoteIPEndPoint);
            sendMsg.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            yield break;
        }
    }
}


