using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;

namespace Robotics.UDP702
{
    /// <summary>
    /// UDP702 contract class
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// DSS contract identifer for UDP702
        /// </summary>
        [DataMember]
        public const string Identifier = "http://schemas.tempuri.org/2011/04/udp702.html";
    }

    /// <summary>
    /// UDP702 state
    /// </summary>
    [DataContract]
    public class UDP702State
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public char Command { get; set; }
        [DataMember]
        public char initChar { get; set; }
        [DataMember]
        public float positionX { get; set; }
        [DataMember]
        public float positionY { get; set; }
    }

    /// <summary>
    /// UDP702 main operations port
    /// </summary>
    [ServicePort]
    public class UDP702Operations : PortSet<DsspDefaultLookup, DsspDefaultDrop, Get, Subscribe, SetCommand, SendMsg, Replace>
    {
    }

    /// <summary>
    /// UDP702 get operation
    /// </summary>
    public class Get : Get<GetRequestType, PortSet<UDP702State, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        public Get()
        {
        }

        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        /// <param name="body">the request message body</param>
        public Get(GetRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Get(GetRequestType body, PortSet<UDP702State, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }

    /// <summary>
    /// UDP702 subscribe operation
    /// </summary>
    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        public Subscribe()
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        public Subscribe(SubscribeRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Subscribe(SubscribeRequestType body, PortSet<SubscribeResponseType, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }
    /// <summary>
    /// UDP002 SetCommand operation
    /// </summary>
    public class SetCommand : Update<SetCommandBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SetCommand() : base(new SetCommandBody()) { }
        public SetCommand(byte[] command) : base(new SetCommandBody(command)) { }
        //        public SetCommand(string command) : base(new SetCommandBody(command)) { }
    }
    // SetCommand Body
    [DataContract]
    public class SetCommandBody
    {
        // RXgN^
        public SetCommandBody() { }
        // RXgN^
        public SetCommandBody(byte[] command)
        //        public SetCommandBody(string command)
        {
            Command = command;
        }
        [DataMember]
        public byte[] Command { get; set; }
        //        public string Command { get; set; }
    }
    /// <summary>
    /// UDP002 SendMsg operation
    /// </summary>
    public class SendMsg : Update<SendMsgBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SendMsg() : base(new SendMsgBody()) { }
        public SendMsg(string msg) : base(new SendMsgBody(msg)) { }
    }
    // SendMsg Body
    [DataContract]
    public class SendMsgBody
    {
        // RXgN^
        public SendMsgBody() { }
        // RXgN^
        public SendMsgBody(string str)
        {
            msgstr = str;
        }
        [DataMember]
        public string msgstr { get; set; }
    }
    /// <summary>
    /// UDP002 Replace operation
    /// </summary>
    public class Replace : Replace<UDP702State, PortSet<DefaultReplaceResponseType, Fault>>
    {
    }
}


