using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;
using submgr = Microsoft.Dss.Services.SubscriptionManager;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Text;

namespace robotics.UDP902
{
    [Contract(Contract.Identifier)]
    [DisplayName("UDP902")]
    [Description("UDP902 service (no description provided)")]
    class UDP902Service : DsspServiceBase
    {
        private const int BUFSIZE = 32;
        private const String robotIP = "192.168.0.2";
        private const int servPort = 8000;
        private const char openCode = 'i';
        private const char packetCode = 'p';
        private const char rotateCode = 'q';
        private const char shutdownCode = 'z';
        byte[] rcvBuffer = new byte[BUFSIZE];
        UdpClient client = null;
        IPEndPoint remoteIPEndPoint = null;
        Thread thread;
        /// <summary>
        /// Service state
        /// </summary>
        [ServiceState]
        UDP902State _state = new UDP902State();

        /// <summary>
        /// Main service port
        /// </summary>
        [ServicePort("/UDP902", AllowMultipleInstances = true)]
        UDP902Operations _mainPort = new UDP902Operations();

        [SubscriptionManagerPartner]
        submgr.SubscriptionManagerPort _submgrPort = new submgr.SubscriptionManagerPort();

        /// <summary>
        /// Service constructor
        /// </summary>
        public UDP902Service(DsspServiceCreationPort creationPort)
            : base(creationPort)
        {
            _state.Name = "fineday";
            _state.Command = 'd';
            _state.Distance = 180.0f;
            _state.Power = 0.5f;
        }

        /// <summary>
        /// Service start
        /// </summary>
        protected override void Start()
        {

            // 
            // Add service specific initialization here
            // 

            base.Start();
            try
            {
                // NCAg̎t
                client = new UdpClient(servPort);
            }
            catch (SocketException se)
            {
                LogInfo(se.ErrorCode + ";" + se.Message);
            }
            // Gh|Cg̍쐬
            remoteIPEndPoint = new IPEndPoint(IPAddress.Any, servPort);

            // NCAg̃ANZX҂
            rcvBuffer = client.Receive(ref remoteIPEndPoint);
            // oCg當֕ϊ
            int bytesRcvd = rcvBuffer.GetLength(0);
            // string֕ϊ
            //           string cmd = Encoding.ASCII.GetString(rcvBuffer, 0, 1);
            char cmd = (char)rcvBuffer[0];
            if (cmd == openCode)
            {
                LogInfo("iPhone accepted");
                // XbhX^[g
                receiveUDP receiveudp = new receiveUDP(remoteIPEndPoint, client, _mainPort);
                thread = new Thread(new ThreadStart(receiveudp.receiveMsg));
                thread.Start();
                LogInfo("thread started");
            }
            else
            {
                LogInfo("unknown code: System CLOSED");
            }
        }

        /// <summary>
        /// Handles Subscribe messages
        /// </summary>
        /// <param name="subscribe">the subscribe request</param>
        [ServiceHandler]
        public void SubscribeHandler(Subscribe subscribe)
        {
            SubscribeHelper(_submgrPort, subscribe.Body, subscribe.ResponsePort);
        }
        /// <summary>
        /// Handles SetCommand messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> SetCommandHandler(SetCommand setCommand)
        {
            rcvBuffer = setCommand.Body.Command;
            char cmd = (char)rcvBuffer[0];
            _state.Command = cmd;
            string s = Encoding.ASCII.GetString(rcvBuffer, 1, 10);
            float t = float.Parse(s);
            _state.Distance = t;
            LogInfo("Distance = " + t);
            string s2 = Encoding.ASCII.GetString(rcvBuffer, 11, 10);
            float n = float.Parse(s2);
            _state.Power = n;
            LogInfo("Power = " + n);
            if (cmd == packetCode)
            {
                LogInfo("packet accepted");

                base.SendNotification(_submgrPort, new Forward(t, n));
            }
            else if (cmd == rotateCode)
            {
                LogInfo("rotateCode accepted");

                base.SendNotification(_submgrPort, new Rotate(t, n));
            }
            else if (cmd == shutdownCode)
            {
                LogInfo("ENDCODE accepted");
                thread.Abort();
                client.Close();
            }
            else
            {
                LogInfo("undefined code: Syetem CLOSED");
                client.Close();
            }
            setCommand.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            yield break;
        }
        /// <summary>
        /// Handles SendMsg messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> SendMsgHandler(SendMsg sendMsg)
        {
            byte[] rcvBuffer = new byte[BUFSIZE];
            string s = System.String.Format("{0,1}{1,10}{2,10}", _state.Command, _state.Distance, _state.Power);
            rcvBuffer = Encoding.ASCII.GetBytes(s);
            client.Send(rcvBuffer, rcvBuffer.Length, remoteIPEndPoint);
            sendMsg.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            yield break;
        }
        /// <summary>
        /// Handles Replace messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> ReplaceHandler(Replace replace)
        {
            _state = replace.Body;
            replace.ResponsePort.Post(DefaultReplaceResponseType.Instance);
            yield break;
        }
    }
}


