using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;

namespace robotics.UDP902
{
    /// <summary>
    /// UDP902 contract class
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// DSS contract identifer for UDP902
        /// </summary>
        [DataMember]
        public const string Identifier = "http://schemas.tempuri.org/2011/04/udp902.html";
    }

    /// <summary>
    /// UDP902 state
    /// </summary>
    [DataContract]
    public class UDP902State
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public char Command { get; set; }
        [DataMember]
        public float Distance { get; set; }
        [DataMember]
        public float Power { get; set; }
    }

    /// <summary>
    /// UDP902 main operations port
    /// </summary>
    [ServicePort]
    public class UDP902Operations : PortSet<DsspDefaultLookup, DsspDefaultDrop, Get, Subscribe, SetCommand, SendMsg, Replace, Rotate, Forward>
    {
    }

    /// <summary>
    /// UDP902 get operation
    /// </summary>
    public class Get : Get<GetRequestType, PortSet<UDP902State, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        public Get()
        {
        }

        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        /// <param name="body">the request message body</param>
        public Get(GetRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Get(GetRequestType body, PortSet<UDP902State, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }

    /// <summary>
    /// UDP902 subscribe operation
    /// </summary>
    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        public Subscribe()
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        public Subscribe(SubscribeRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Subscribe(SubscribeRequestType body, PortSet<SubscribeResponseType, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }
    /// <summary>
    /// UDP SetCommand operation
    /// </summary>
    public class SetCommand : Update<SetCommandBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SetCommand() : base(new SetCommandBody()) { }
        public SetCommand(byte[] command) : base(new SetCommandBody(command)) { }
    }
    // SetCommand Body
    [DataContract]
    public class SetCommandBody
    {
        // RXgN^
        public SetCommandBody() { }
        // RXgN^
        public SetCommandBody(byte[] command)
        {
            Command = command;
        }
        [DataMember]
        public byte[] Command { get; set; }
    }
    /// <summary>
    /// UDP SendMsg operation
    /// </summary>
    public class SendMsg : Update<SendMsgBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SendMsg() : base(new SendMsgBody()) { }
        public SendMsg(string msg) : base(new SendMsgBody(msg)) { }
    }
    // SendMsg Body
    [DataContract]
    public class SendMsgBody
    {
        // RXgN^
        public SendMsgBody() { }
        // RXgN^
        public SendMsgBody(string str)
        {
            msgstr = str;
        }
        [DataMember]
        public string msgstr { get; set; }
    }
    /// <summary>
    /// UDP Rotate operation
    /// </summary>
    public class Rotate : Update<RotateBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public Rotate() : base(new RotateBody()) { }
        public Rotate(float degrees, float power) : base(new RotateBody(degrees, power)) { }
    }
    // Rotate Body
    [DataContract]
    public class RotateBody
    {
        // RXgN^
        public RotateBody() { }
        // RXgN^
        public RotateBody(float degrees, float power)
        {
            Degrees = degrees;
            Power = power;
        }
        [DataMember]
        public float Degrees { get; set; }
        [DataMember]
        public float Power { get; set; }
    }
    /// <summary>
    /// UDP Forward operation
    /// </summary>
    public class Forward : Update<ForwardBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public Forward() : base(new ForwardBody()) { }
        public Forward(float distance, float power) : base(new ForwardBody(distance, power)) { }
    }
    // Forward Body
    [DataContract]
    public class ForwardBody
    {
        // RXgN^
        public ForwardBody() { }
        // RXgN^
        public ForwardBody(float distance, float power)
        {
            Distance = distance;
            Power = power;
        }
        [DataMember]
        public float Distance { get; set; }
        [DataMember]
        public float Power { get; set; }
    }
    /// <summary>
    /// UDP002 Replace operation
    /// </summary>
    public class Replace : Replace<UDP902State, PortSet<DefaultReplaceResponseType, Fault>>
    {
    }
}


