using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;

namespace Robotics.TCP1002
{
    /// <summary>
    /// TCP1002 contract class
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// DSS contract identifer for TCP1002
        /// </summary>
        [DataMember]
        public const string Identifier = "http://schemas.tempuri.org/2011/04/tcp1002.html";
    }

    /// <summary>
    /// TCP1002 state
    /// </summary>
    [DataContract]
    public class TCP1002State
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public char Command { get; set; }
        [DataMember]
        public float rightPower { get; set; }
        [DataMember]
        public float leftPower { get; set; }
    }

    /// <summary>
    /// TCP1002 main operations port
    /// </summary>
    [ServicePort]
    public class TCP1002Operations : PortSet<DsspDefaultLookup, DsspDefaultDrop, Get, Subscribe, SetCommand, SendMsg, Replace, SetDrivePower>
    {
    }

    /// <summary>
    /// TCP1002 get operation
    /// </summary>
    public class Get : Get<GetRequestType, PortSet<TCP1002State, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        public Get()
        {
        }

        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        /// <param name="body">the request message body</param>
        public Get(GetRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Get(GetRequestType body, PortSet<TCP1002State, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }

    /// <summary>
    /// TCP1002 subscribe operation
    /// </summary>
    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        public Subscribe()
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        public Subscribe(SubscribeRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Subscribe(SubscribeRequestType body, PortSet<SubscribeResponseType, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }
    /// <summary>
    /// TCP SetCommand operation
    /// </summary>
    public class SetCommand : Update<SetCommandBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SetCommand() : base(new SetCommandBody()) { }
        public SetCommand(byte[] command) : base(new SetCommandBody(command)) { }
    }
    [DataContract]
    public class SetCommandBody
    {
        // RXgN^
        public SetCommandBody()
        {
        }
        // RXgN^
        public SetCommandBody(byte[] command)
        {
            Command = command;
        }
        [DataMember]
        public byte[] Command { get; set; }
    }
    /// <summary>
    /// TCP SendMsg operation
    /// </summary>
    public class SendMsg : Update<SendMsgBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SendMsg() : base(new SendMsgBody()) { }
        public SendMsg(string msg) : base(new SendMsgBody(msg)) { }
    }
    // SendMsg Body
    [DataContract]
    public class SendMsgBody
    {
        // RXgN^
        public SendMsgBody() { }
        // RXgN^
        public SendMsgBody(string str)
        {
            msgstr = str;
        }
        [DataMember]
        public string msgstr { get; set; }
    }
    /// <summary>
    /// UDP SetDrivePower operation
    /// </summary>
    public class SetDrivePower : Update<SetDrivePowerBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SetDrivePower() : base(new SetDrivePowerBody()) { }
        public SetDrivePower(float rightPower, float leftPower) : base(new SetDrivePowerBody(rightPower, leftPower)) { }
    }
    // SetDrivePower Body
    [DataContract]
    public class SetDrivePowerBody
    {
        // RXgN^
        public SetDrivePowerBody() { }
        // RXgN^
        public SetDrivePowerBody(float rightPower, float leftPower)
        {
            RightPower = rightPower;
            LeftPower = leftPower;
        }
        [DataMember]
        public float RightPower { get; set; }
        [DataMember]
        public float LeftPower { get; set; }
    }
    /// <summary>
    /// UDP Replace operation
    /// </summary>
    public class Replace : Replace<TCP1002State, PortSet<DefaultReplaceResponseType, Fault>>
    {
    }
}


