using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;
using submgr = Microsoft.Dss.Services.SubscriptionManager;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Text;

namespace Robotics.TCP1003
{
    [Contract(Contract.Identifier)]
    [DisplayName("TCP1003")]
    [Description("TCP1003 service (no description provided)")]
    class TCP1003Service : DsspServiceBase
    {
        private const int BUFSIZE = 32;
        private const String robotIP = "192.168.0.2";
        private const int servPort = 8000;
        private const char openCode = 'i';
        private const char packetCode = 'p';
        private const char shutdownCode = 'z';
        byte[] rcvBuffer = new byte[BUFSIZE];

        private Socket clntSock;
        Thread thread;
        /// <summary>
        /// Service state
        /// </summary>
        [ServiceState]
        TCP1003State _state = new TCP1003State();

        /// <summary>
        /// Main service port
        /// </summary>
        [ServicePort("/TCP1003", AllowMultipleInstances = true)]
        TCP1003Operations _mainPort = new TCP1003Operations();

        [SubscriptionManagerPartner]
        submgr.SubscriptionManagerPort _submgrPort = new submgr.SubscriptionManagerPort();

        /// <summary>
        /// Service constructor
        /// </summary>
        public TCP1003Service(DsspServiceCreationPort creationPort)
            : base(creationPort)
        {
            _state.Name = "fineday";
            _state.Command = '\n';
            _state.Power = 0.0f;
            _state.Angle = 0.0f;
        }

        /// <summary>
        /// Service start
        /// </summary>
        protected override void Start()
        {

            // 
            // Add service specific initialization here
            // 

            base.Start();
            // bXJn
            TcpListener listener = null;
            listener = new TcpListener(IPAddress.Any, servPort);
            listener.Start();

            // ANZvg
            clntSock = listener.AcceptSocket();
            LogInfo("iPhone accepted");

            // XbhX^[g
            receiveTCP receivetcp = new receiveTCP(clntSock, _mainPort);
            thread = new Thread(new ThreadStart(receivetcp.receiveMsg));
            thread.Start();
        }

        /// <summary>
        /// Handles Subscribe messages
        /// </summary>
        /// <param name="subscribe">the subscribe request</param>
        [ServiceHandler]
        public void SubscribeHandler(Subscribe subscribe)
        {
            SubscribeHelper(_submgrPort, subscribe.Body, subscribe.ResponsePort);
        }
        /// <summary>
        /// Handles SetCommand messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> SetCommandHandler(SetCommand setCommand)
        {
            rcvBuffer = setCommand.Body.Command;
            char cmd = (char)rcvBuffer[0];
            _state.Command = cmd;
            string s = Encoding.ASCII.GetString(rcvBuffer, 1, 10);
            float t = float.Parse(s);
            LogInfo("time = " + t);
            string s2 = Encoding.ASCII.GetString(rcvBuffer, 11, 10);
            float n = float.Parse(s2);
            LogInfo("number = " + n);
            if (cmd == packetCode)
            {
                LogInfo("packet accepted");
                base.SendNotification(_submgrPort, new SetDrive(t, n));
            }
            else if (cmd == shutdownCode)
            {
                LogInfo("ENDCODE accepted");
                thread.Abort();
            }
            else
            {
                LogInfo("undefined code: Syetem CLOSED");
            }
            setCommand.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            yield break;
        }
        /// <summary>
        /// Handles SendMsg messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> SendMsgHandler(SendMsg sendMsg)
        {
            string s = System.String.Format("{0,1}{1,10}{2,10}", _state.Command, _state.Power, _state.Angle);
            rcvBuffer = Encoding.ASCII.GetBytes(s);
            clntSock.Send(rcvBuffer, 0, rcvBuffer.Length, SocketFlags.None);
            sendMsg.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            yield break;
        }
        /// <summary>
        /// Handles Replace messages
        /// </summary>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive)]
        public IEnumerator<ITask> ReplaceHandler(Replace replace)
        {
            _state = replace.Body;
            replace.ResponsePort.Post(DefaultReplaceResponseType.Instance);
            yield break;
        }
    }
}


