using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;

namespace Robotics.TCP1003
{
    /// <summary>
    /// TCP1003 contract class
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// DSS contract identifer for TCP1003
        /// </summary>
        [DataMember]
        public const string Identifier = "http://schemas.tempuri.org/2011/04/tcp1003.html";
    }

    /// <summary>
    /// TCP1003 state
    /// </summary>
    [DataContract]
    public class TCP1003State
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public char Command { get; set; }
        [DataMember]
        public float Power { get; set; }
        [DataMember]
        public float Angle { get; set; }
    }

    /// <summary>
    /// TCP1003 main operations port
    /// </summary>
    [ServicePort]
    public class TCP1003Operations : PortSet<DsspDefaultLookup, DsspDefaultDrop, Get, Subscribe, SetCommand, SendMsg, Replace, SetDrive>
    {
    }

    /// <summary>
    /// TCP1003 get operation
    /// </summary>
    public class Get : Get<GetRequestType, PortSet<TCP1003State, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        public Get()
        {
        }

        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        /// <param name="body">the request message body</param>
        public Get(GetRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Get
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Get(GetRequestType body, PortSet<TCP1003State, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }

    /// <summary>
    /// TCP1003 subscribe operation
    /// </summary>
    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        public Subscribe()
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        public Subscribe(SubscribeRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Subscribe(SubscribeRequestType body, PortSet<SubscribeResponseType, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }
    /// <summary>
    /// TCP SetCommand operation
    /// </summary>
    public class SetCommand : Update<SetCommandBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SetCommand() : base(new SetCommandBody()) { }
        public SetCommand(byte[] command) : base(new SetCommandBody(command)) { }
    }
    [DataContract]
    public class SetCommandBody
    {
        // RXgN^
        public SetCommandBody()
        {
        }
        // RXgN^
        public SetCommandBody(byte[] command)
        {
            Command = command;
        }
        [DataMember]
        public byte[] Command { get; set; }
    }
    /// <summary>
    /// TCP SendMsg operation
    /// </summary>
    public class SendMsg : Update<SendMsgBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SendMsg() : base(new SendMsgBody()) { }
        public SendMsg(string msg) : base(new SendMsgBody(msg)) { }
    }
    // SendMsg Body
    [DataContract]
    public class SendMsgBody
    {
        // RXgN^
        public SendMsgBody() { }
        // RXgN^
        public SendMsgBody(string str)
        {
            msgstr = str;
        }
        [DataMember]
        public string msgstr { get; set; }
    }
    /// <summary>
    /// UDP SetDrive operation
    /// </summary>
    public class SetDrive : Update<SetDriveBody, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public SetDrive() : base(new SetDriveBody()) { }
        public SetDrive(float power, float angle) : base(new SetDriveBody(power, angle)) { }
    }
    // SetDrivePower Body
    [DataContract]
    public class SetDriveBody
    {
        // RXgN^
        public SetDriveBody() { }
        // RXgN^
        public SetDriveBody(float power, float angle)
        {
            Power = power;
            Angle = angle;
        }
        [DataMember]
        public float Power { get; set; }
        [DataMember]
        public float Angle { get; set; }
    }
    /// <summary>
    /// UDP Replace operation
    /// </summary>
    public class Replace : Replace<TCP1003State, PortSet<DefaultReplaceResponseType, Fault>>
    {
    }
}


